@@ -168,11 +168,9 @@ void check_polygon(drawvec &geom) {
 	}
 }
 
-drawvec reduce_tiny_poly(drawvec const &geom, int z, int detail, bool *still_needs_simplification, bool *reduced_away, double *accum_area, serial_feature *this_feature, serial_feature *tiny_feature) {
+drawvec reduce_tiny_poly(drawvec const &geom, int z, int detail, bool *still_needs_simplification, bool *reduced_away, double *accum_area) {
 	drawvec out;
 	const double pixel = (1LL << (32 - detail - z)) * (double) tiny_polygon_size;
-	bool includes_real = false;
-	bool includes_dust = false;
 
 	bool included_last_outer = false;
 	*still_needs_simplification = false;
@@ -217,7 +215,6 @@ drawvec reduce_tiny_poly(drawvec const &geom, int z, int detail, bool *still_nee
 						out.emplace_back(VT_LINETO, geom[i].x - pixel / 2 + pixel, geom[i].y - pixel / 2 + pixel);
 						out.emplace_back(VT_LINETO, geom[i].x - pixel / 2, geom[i].y - pixel / 2 + pixel);
 						out.emplace_back(VT_LINETO, geom[i].x - pixel / 2, geom[i].y - pixel / 2);
-						includes_dust = true;
 
 						*accum_area -= pixel * pixel;
 					}
@@ -238,7 +235,6 @@ drawvec reduce_tiny_poly(drawvec const &geom, int z, int detail, bool *still_nee
 					// which means that the overall polygon has a real geometry,
 					// which means that it gets to be simplified.
 					*still_needs_simplification = true;
-					includes_real = true;
 
 					if (area > 0) {
 						included_last_outer = true;
@@ -260,28 +256,6 @@ drawvec reduce_tiny_poly(drawvec const &geom, int z, int detail, bool *still_nee
 			fprintf(stderr, "\n");
 
 			out.push_back(geom[i]);
-			includes_real = true;
-		}
-	}
-
-	if (!includes_real) {
-		if (includes_dust) {
-			// this geometry is just dust, so if there is another feature that
-			// contributed to the dust that is larger than this feature,
-			// keep its attributes instead of this one that just happened to be
-			// the one that hit the threshold of survival.
-
-			if (tiny_feature->extent > this_feature->extent) {
-				*this_feature = *tiny_feature;
-				tiny_feature->extent = 0;
-			}
-		} else {
-			// this is a feature that we are throwing away, so hang on to it
-			// attributes if it is bigger than the biggest one we threw away so far
-
-			if (this_feature->extent > tiny_feature->extent) {
-				*tiny_feature = *this_feature;
-			}
 		}
 	}
 
@@ -329,8 +303,15 @@ bool point_within_tile(long long x, long long y, int z) {
 double distance_from_line(long long point_x, long long point_y, long long segA_x, long long segA_y, long long segB_x, long long segB_y) {
 	long long p2x = segB_x - segA_x;
 	long long p2y = segB_y - segA_y;
-	double something = p2x * p2x + p2y * p2y;
-	double u = (0 == something) ? 0 : ((point_x - segA_x) * p2x + (point_y - segA_y) * p2y) / (something);
+
+	// These calculations must be made in integers instead of floating point
+	// to make them consistent between x86 and arm floating point implementations.
+	//
+	// Coordinates may be up to 34 bits, so their product is up to 68 bits,
+	// making their sum up to 69 bits. Downshift before multiplying to keep them in range.
+	double something = ((p2x / 4) * (p2x / 8) + (p2y / 4) * (p2y / 8)) * 32.0;
+	// likewise
+	double u = (0 == something) ? 0 : ((point_x - segA_x) / 4 * (p2x / 8) + (point_y - segA_y) / 4 * (p2y / 8)) * 32.0 / (something);
 
 	if (u >= 1) {
 		u = 1;
@@ -1215,7 +1196,7 @@ drawvec polygon_to_anchor(const drawvec &geom) {
 				std::vector<candidate> candidates;
 
 				for (size_t pass = 0; pass < 4; pass++) {
-					std::sort(points.begin(), points.end(), sorty_sorter(pass));
+					std::stable_sort(points.begin(), points.end(), sorty_sorter(pass));
 
 					for (size_t i = 1; i < points.size(); i++) {
 						double dx = points[i].x - points[i - 1].x;
@@ -1239,7 +1220,7 @@ drawvec polygon_to_anchor(const drawvec &geom) {
 				// segment, if we find one whose midpoint is inside the polygon and
 				// far enough from any edge to be good enough, stop looking.
 
-				std::sort(candidates.begin(), candidates.end());
+				std::stable_sort(candidates.begin(), candidates.end());
 				// only check the top 50 stride midpoints, since this list can be quite large
 				for (size_t i = 0; i < candidates.size() && i < 50; i++) {
 					double maybe_goodness = label_goodness(geom, candidates[i].x, candidates[i].y);
